@@ -16,22 +16,22 @@ Struct::Path::PerlStyle - Perl-style syntax frontend for L<Struct::Path|Struct::
 
 =head1 VERSION
 
-Version 0.20
+Version 0.22
 
 =cut
 
-our $VERSION = '0.20';
+our $VERSION = '0.22';
 
 =head1 SYNOPSIS
 
     use Struct::Path::PerlStyle qw(ps_parse ps_serialize);
 
-    $struct = ps_parse('{a}{b}[1]');    # L<Struct::Path|Struct::Path> compatible
+    $struct = ps_parse('{a}{b}[1]');    # Struct::Path compatible
     $string = ps_serialize($struct);    # convert Struct::Path path to string
 
 =head1 EXPORT
 
-Nothing exports by default.
+Nothing is exported by default.
 
 =head1 SUBROUTINES
 
@@ -46,7 +46,8 @@ Path syntax examples:
     "{a}{b}"              # means b's value
     "{a}{}"               # all values from a's subhash; same for arrays (using empty square brackets)
     "{a}{b,c}"            # b's and c's values
-    "{a}{b c}"            # same, space is also a delimiter (except if quoted)
+    "{a}{b c}"            # same, space is also a delimiter
+    "{a}{'space inside'}" # keys with spaces/tabs must be quoted (double quotes supported as well)
     "{a}{b}[0,1,2,5]"     # 0, 1, 2 and 5 array's items
     "{a}{b}[0..2,5]"      # same, but using ranges
     "{a}{b}[9..0]"        # descending ranges allowed (perl doesn't)
@@ -66,8 +67,8 @@ sub ps_parse($) {
         for my $item ($step->elements) {
             $item->prune('PPI::Token::Whitespace') if $item->can('prune');
 
-            if ($item->isa('PPI::Structure::Block') or
-                ($item->isa('PPI::Structure::Constructor') and $item->first_token->content eq '{')) {
+            if ($item->isa('PPI::Structure::Constructor') and $item->first_token->content eq '{' or
+                $item->isa('PPI::Structure::Block')) {
                 push @{$out}, {};
                 for my $t (map { $_->elements } $item->children) {
                     my $key;
@@ -82,11 +83,13 @@ sub ps_parse($) {
                     }
                     push @{$out->[-1]->{keys}}, $key;
                 }
-            } elsif ($item->isa('PPI::Structure::Constructor') and $item->first_token->content eq '[') {
+            } elsif ($item->isa('PPI::Structure::Constructor')) { # PPI::Structure::Constructor is hash/array constructir only
                 push @{$out}, [];
                 my $is_range;
                 for my $t (map { $_->elements } $item->children) {
                     if ($t->isa('PPI::Token::Number')) {
+                        croak "Floating-point numbers not allowed as array indexes (step #$sc)"
+                            unless ($t->content == int($t->content));
                         if ($is_range) {
                             my $start = pop(@{$out->[-1]});
                             croak "Undefined start for range (step #$sc)" unless (defined $start);
@@ -94,7 +97,7 @@ sub ps_parse($) {
                                 ($start < $t->content ? $start..$t->content : reverse $t->content..$start);
                             $is_range = undef;
                         } else {
-                            push @{$out->[-1]}, $t->content + 0;
+                            push @{$out->[-1]}, int($t->content);
                         }
                     } elsif ($t->isa('PPI::Token::Operator') and $t->content eq ',') {
                         $is_range = undef;
@@ -148,8 +151,9 @@ sub ps_serialize($) {
             $out .= "[" . join(",", map { $_->[0] != $_->[1] ? "$_->[0]..$_->[1]" : $_->[0] } @{ranges}) . "]";
         } elsif (ref $step eq 'HASH') {
             croak "Unsupported hash definition (step #$sc)"
-                if (keys %{$step} and (not exists $step->{keys} or ref $step->{keys} ne 'ARRAY'));
-            $out .= "{" . join(",", exists $step->{keys} ? @{$step->{keys}} : "") . "}";
+                if (keys %{$step} and not (exists $step->{keys} and ref $step->{keys} eq 'ARRAY'));
+            my @items = exists $step->{keys} ? map { split(/\s/, $_, -1) > 1 ? "'$_'" : $_ } @{$step->{keys}} : "";
+            $out .= "{" . join(",", @items) . "}";
         } else {
             croak "Unsupported thing in the path (step #$sc)";
         }
